@@ -26,19 +26,24 @@
  */
 
 #include "avcodec.h"
+#include "bytestream.h"
+#include "config.h"
 #if CONFIG_ZLIB
 #include <zlib.h>
 #endif
 #include "lzw.h"
 #include "tiff.h"
 #include "faxcompr.h"
-#include "libavutil/common.h"
+#include "internal.h"
+#include "mathops.h"
+#include "libavutil/attributes.h"
 #include "libavutil/intreadwrite.h"
 #include "libavutil/imgutils.h"
 
 typedef struct TiffContext {
     AVCodecContext *avctx;
     AVFrame picture;
+    GetByteContext gb;
 
     int width, height;
     unsigned int bpp, bppcount;
@@ -53,40 +58,37 @@ typedef struct TiffContext {
 
     int strips, rps, sstype;
     int sot;
-    const uint8_t* stripdata;
-    const uint8_t* stripsizes;
-    int stripsize, stripoff;
+    int stripsizesoff, stripsize, stripoff, strippos;
     LZWState *lzw;
 } TiffContext;
 
-static unsigned tget_short(const uint8_t **p, int le) {
-    unsigned v = le ? AV_RL16(*p) : AV_RB16(*p);
-    *p += 2;
-    return v;
+static unsigned tget_short(GetByteContext *gb, int le)
+{
+    return le ? bytestream2_get_le16(gb) : bytestream2_get_be16(gb);
 }
 
-static unsigned tget_long(const uint8_t **p, int le) {
-    unsigned v = le ? AV_RL32(*p) : AV_RB32(*p);
-    *p += 4;
-    return v;
+static unsigned tget_long(GetByteContext *gb, int le)
+{
+    return le ? bytestream2_get_le32(gb) : bytestream2_get_be32(gb);
 }
 
-static unsigned tget(const uint8_t **p, int type, int le) {
-    switch(type){
-    case TIFF_BYTE : return *(*p)++;
-    case TIFF_SHORT: return tget_short(p, le);
-    case TIFF_LONG : return tget_long (p, le);
-    default        : return UINT_MAX;
+static unsigned tget(GetByteContext *gb, int type, int le)
+{
+    switch (type) {
+    case TIFF_BYTE:  return bytestream2_get_byte(gb);
+    case TIFF_SHORT: return tget_short(gb, le);
+    case TIFF_LONG:  return tget_long(gb, le);
+    default:         return UINT_MAX;
     }
 }
 
 #if CONFIG_ZLIB
-static int tiff_uncompress(uint8_t *dst, unsigned long *len, const uint8_t *src, int size)
+static int tiff_uncompress(uint8_t *dst, unsigned long *len, const uint8_t *src,
+                           int size)
 {
-    z_stream zstream;
+    z_stream zstream = { 0 };
     int zret;
 
-    memset(&zstream, 0, sizeof(zstream));
     zstream.next_in = src;
     zstream.avail_in = size;
     zstream.next_out = dst;
@@ -103,25 +105,35 @@ static int tiff_uncompress(uint8_t *dst, unsigned long *len, const uint8_t *src,
 }
 #endif
 
-static int tiff_unpack_strip(TiffContext *s, uint8_t* dst, int stride, const uint8_t *src, int size, int lines){
+static int tiff_unpack_strip(TiffContext *s, uint8_t *dst, int stride,
+                             const uint8_t *src, int size, int lines)
+{
+    PutByteContext pb;
     int c, line, pixels, code;
-    const uint8_t *ssrc = src;
     int width = ((s->width * s->bpp) + 7) >> 3;
-#if CONFIG_ZLIB
-    uint8_t *zbuf; unsigned long outlen;
 
-    if(s->compr == TIFF_DEFLATE || s->compr == TIFF_ADOBE_DEFLATE){
+    if (size <= 0)
+        return AVERROR_INVALIDDATA;
+
+#if CONFIG_ZLIB
+    if (s->compr == TIFF_DEFLATE || s->compr == TIFF_ADOBE_DEFLATE) {
+        uint8_t *zbuf;
+        unsigned long outlen;
         int ret;
         outlen = width * lines;
         zbuf = av_malloc(outlen);
+        if (!zbuf)
+            return AVERROR(ENOMEM);
         ret = tiff_uncompress(zbuf, &outlen, src, size);
-        if(ret != Z_OK){
-            av_log(s->avctx, AV_LOG_ERROR, "Uncompressing failed (%lu of %lu) with error %d\n", outlen, (unsigned long)width * lines, ret);
+        if (ret != Z_OK) {
+            av_log(s->avctx, AV_LOG_ERROR,
+                   "Uncompressing failed (%lu of %lu) with error %d\n", outlen,
+                   (unsigned long)width * lines, ret);
             av_free(zbuf);
             return -1;
         }
         src = zbuf;
-        for(line = 0; line < lines; line++){
+        for (line = 0; line < lines; line++) {
             memcpy(dst, src, width);
             dst += stride;
             src += width;
@@ -130,96 +142,91 @@ static int tiff_unpack_strip(TiffContext *s, uint8_t* dst, int stride, const uin
         return 0;
     }
 #endif
-    if(s->compr == TIFF_LZW){
-        if(ff_lzw_decode_init(s->lzw, 8, src, size, FF_LZW_TIFF) < 0){
+    if (s->compr == TIFF_LZW) {
+        if (ff_lzw_decode_init(s->lzw, 8, src, size, FF_LZW_TIFF) < 0) {
             av_log(s->avctx, AV_LOG_ERROR, "Error initializing LZW decoder\n");
             return -1;
         }
-    }
-    if(s->compr == TIFF_CCITT_RLE || s->compr == TIFF_G3 || s->compr == TIFF_G4){
-        int i, ret = 0;
-        uint8_t *src2 = av_malloc(size + FF_INPUT_BUFFER_PADDING_SIZE);
-
-        if(!src2 || (unsigned)size + FF_INPUT_BUFFER_PADDING_SIZE < (unsigned)size){
-            av_log(s->avctx, AV_LOG_ERROR, "Error allocating temporary buffer\n");
-            return -1;
+        for (line = 0; line < lines; line++) {
+            pixels = ff_lzw_decode(s->lzw, dst, width);
+            if (pixels < width) {
+                av_log(s->avctx, AV_LOG_ERROR, "Decoded only %i bytes of %i\n",
+                       pixels, width);
+                return AVERROR_INVALIDDATA;
+            }
+            dst += stride;
         }
-        if(s->fax_opts & 2){
-            av_log(s->avctx, AV_LOG_ERROR, "Uncompressed fax mode is not supported (yet)\n");
+        return 0;
+    }
+    if (s->compr == TIFF_CCITT_RLE || s->compr == TIFF_G3
+        || s->compr == TIFF_G4) {
+        int i, ret = 0;
+        uint8_t *src2 = av_malloc((unsigned)size +
+                                  FF_INPUT_BUFFER_PADDING_SIZE);
+
+        if (!src2) {
+            av_log(s->avctx, AV_LOG_ERROR,
+                   "Error allocating temporary buffer\n");
+            return AVERROR(ENOMEM);
+        }
+        if (s->fax_opts & 2) {
+            av_log(s->avctx, AV_LOG_ERROR,
+                   "Uncompressed fax mode is not supported (yet)\n");
             av_free(src2);
             return -1;
         }
-        if(!s->fill_order){
+        if (!s->fill_order) {
             memcpy(src2, src, size);
-        }else{
-            for(i = 0; i < size; i++)
-                src2[i] = av_reverse[src[i]];
+        } else {
+            for (i = 0; i < size; i++)
+                src2[i] = ff_reverse[src[i]];
         }
-        memset(src2+size, 0, FF_INPUT_BUFFER_PADDING_SIZE);
-        switch(s->compr){
+        memset(src2 + size, 0, FF_INPUT_BUFFER_PADDING_SIZE);
+        switch (s->compr) {
         case TIFF_CCITT_RLE:
         case TIFF_G3:
         case TIFF_G4:
-            ret = ff_ccitt_unpack(s->avctx, src2, size, dst, lines, stride, s->compr, s->fax_opts);
+            ret = ff_ccitt_unpack(s->avctx, src2, size, dst, lines, stride,
+                                  s->compr, s->fax_opts);
             break;
         }
         av_free(src2);
         return ret;
     }
-    for(line = 0; line < lines; line++){
-        if(src - ssrc > size){
-            av_log(s->avctx, AV_LOG_ERROR, "Source data overread\n");
-            return -1;
-        }
-        switch(s->compr){
+
+    bytestream2_init(&s->gb, src, size);
+    bytestream2_init_writer(&pb, dst, stride * lines);
+
+    for (line = 0; line < lines; line++) {
+        if (bytestream2_get_bytes_left(&s->gb) == 0 || bytestream2_get_eof(&pb))
+            break;
+        bytestream2_seek_p(&pb, stride * line, SEEK_SET);
+        switch (s->compr) {
         case TIFF_RAW:
-            if (ssrc + size - src < width)
-                return AVERROR_INVALIDDATA;
             if (!s->fill_order) {
-                memcpy(dst, src, width);
+                bytestream2_copy_buffer(&pb, &s->gb, width);
             } else {
                 int i;
                 for (i = 0; i < width; i++)
-                    dst[i] = av_reverse[src[i]];
+                    bytestream2_put_byte(&pb, ff_reverse[bytestream2_get_byte(&s->gb)]);
             }
-            src += width;
             break;
         case TIFF_PACKBITS:
-            for(pixels = 0; pixels < width;){
-                if (ssrc + size - src < 2)
-                    return AVERROR_INVALIDDATA;
-                code = (int8_t)*src++;
-                if(code >= 0){
+            for (pixels = 0; pixels < width;) {
+                code = (int8_t)bytestream2_get_byte(&s->gb);
+                if (code >= 0) {
                     code++;
-                    if (pixels + code > width ||
-                        ssrc + size - src < code) {
-                        av_log(s->avctx, AV_LOG_ERROR, "Copy went out of bounds\n");
-                        return -1;
-                    }
-                    memcpy(dst + pixels, src, code);
-                    src += code;
+                    bytestream2_copy_buffer(&pb, &s->gb, code);
                     pixels += code;
-                }else if(code != -128){ // -127..-1
+                } else if (code != -128) { // -127..-1
                     code = (-code) + 1;
-                    if(pixels + code > width){
-                        av_log(s->avctx, AV_LOG_ERROR, "Run went out of bounds\n");
-                        return -1;
-                    }
-                    c = *src++;
-                    memset(dst + pixels, c, code);
+                    c    = bytestream2_get_byte(&s->gb);
+                    bytestream2_set_buffer(&pb, c, code);
                     pixels += code;
                 }
             }
             break;
-        case TIFF_LZW:
-            pixels = ff_lzw_decode(s->lzw, dst, width);
-            if(pixels < width){
-                av_log(s->avctx, AV_LOG_ERROR, "Decoded only %i bytes of %i\n", pixels, width);
-                return -1;
-            }
-            break;
         }
-        dst += stride;
     }
     return 0;
 }
@@ -231,22 +238,22 @@ static int init_image(TiffContext *s)
 
     switch (s->bpp * 10 + s->bppcount) {
     case 11:
-        s->avctx->pix_fmt = PIX_FMT_MONOBLACK;
+        s->avctx->pix_fmt = AV_PIX_FMT_MONOBLACK;
         break;
     case 81:
-        s->avctx->pix_fmt = PIX_FMT_PAL8;
+        s->avctx->pix_fmt = AV_PIX_FMT_PAL8;
         break;
     case 243:
-        s->avctx->pix_fmt = PIX_FMT_RGB24;
+        s->avctx->pix_fmt = AV_PIX_FMT_RGB24;
         break;
     case 161:
-        s->avctx->pix_fmt = PIX_FMT_GRAY16BE;
+        s->avctx->pix_fmt = s->le ? AV_PIX_FMT_GRAY16LE : AV_PIX_FMT_GRAY16BE;
         break;
     case 324:
-        s->avctx->pix_fmt = PIX_FMT_RGBA;
+        s->avctx->pix_fmt = AV_PIX_FMT_RGBA;
         break;
     case 483:
-        s->avctx->pix_fmt = s->le ? PIX_FMT_RGB48LE : PIX_FMT_RGB48BE;
+        s->avctx->pix_fmt = s->le ? AV_PIX_FMT_RGB48LE : AV_PIX_FMT_RGB48BE;
         break;
     default:
         av_log(s->avctx, AV_LOG_ERROR,
@@ -261,11 +268,11 @@ static int init_image(TiffContext *s)
     }
     if (s->picture.data[0])
         s->avctx->release_buffer(s->avctx, &s->picture);
-    if ((ret = s->avctx->get_buffer(s->avctx, &s->picture)) < 0) {
+    if ((ret = ff_get_buffer(s->avctx, &s->picture)) < 0) {
         av_log(s->avctx, AV_LOG_ERROR, "get_buffer() failed\n");
         return ret;
     }
-    if (s->avctx->pix_fmt == PIX_FMT_PAL8) {
+    if (s->avctx->pix_fmt == AV_PIX_FMT_PAL8) {
         if (s->palette_is_set) {
             memcpy(s->picture.data[1], s->palette, sizeof(s->palette));
         } else {
@@ -278,60 +285,53 @@ static int init_image(TiffContext *s)
     return 0;
 }
 
-static int tiff_decode_tag(TiffContext *s, const uint8_t *start, const uint8_t *buf, const uint8_t *end_buf)
+static int tiff_decode_tag(TiffContext *s)
 {
     unsigned tag, type, count, off, value = 0;
-    int i, j;
+    int i, start;
     uint32_t *pal;
-    const uint8_t *rp, *gp, *bp;
 
-    if (end_buf - buf < 12)
+    if (bytestream2_get_bytes_left(&s->gb) < 12)
         return -1;
-    tag = tget_short(&buf, s->le);
-    type = tget_short(&buf, s->le);
-    count = tget_long(&buf, s->le);
-    off = tget_long(&buf, s->le);
+    tag   = tget_short(&s->gb, s->le);
+    type  = tget_short(&s->gb, s->le);
+    count = tget_long(&s->gb, s->le);
+    off   = tget_long(&s->gb, s->le);
+    start = bytestream2_tell(&s->gb);
 
     if (type == 0 || type >= FF_ARRAY_ELEMS(type_sizes)) {
-        av_log(s->avctx, AV_LOG_DEBUG, "Unknown tiff type (%u) encountered\n", type);
+        av_log(s->avctx, AV_LOG_DEBUG, "Unknown tiff type (%u) encountered\n",
+               type);
         return 0;
     }
 
-    if(count == 1){
-        switch(type){
+    if (count == 1) {
+        switch (type) {
         case TIFF_BYTE:
         case TIFF_SHORT:
-            buf -= 4;
-            value = tget(&buf, type, s->le);
-            buf = NULL;
+            bytestream2_seek(&s->gb, -4, SEEK_CUR);
+            value = tget(&s->gb, type, s->le);
             break;
         case TIFF_LONG:
             value = off;
-            buf = NULL;
             break;
         case TIFF_STRING:
-            if(count <= 4){
-                buf -= 4;
+            if (count <= 4) {
+                bytestream2_seek(&s->gb, -4, SEEK_CUR);
                 break;
             }
         default:
             value = UINT_MAX;
-            buf = start + off;
+            bytestream2_seek(&s->gb, off, SEEK_SET);
         }
     } else {
-        if (count <= 4 && type_sizes[type] * count <= 4) {
-            buf -= 4;
-        } else {
-            buf = start + off;
-        }
+        if (count <= 4 && type_sizes[type] * count <= 4)
+            bytestream2_seek(&s->gb, -4, SEEK_CUR);
+        else
+            bytestream2_seek(&s->gb, off, SEEK_SET);
     }
 
-    if(buf && (buf < start || buf > end_buf)){
-        av_log(s->avctx, AV_LOG_ERROR, "Tag referencing position outside the image\n");
-        return -1;
-    }
-
-    switch(tag){
+    switch (tag) {
     case TIFF_WIDTH:
         s->width = value;
         break;
@@ -340,20 +340,25 @@ static int tiff_decode_tag(TiffContext *s, const uint8_t *start, const uint8_t *
         break;
     case TIFF_BPP:
         s->bppcount = count;
-        if(count > 4){
-            av_log(s->avctx, AV_LOG_ERROR, "This format is not supported (bpp=%d, %d components)\n", s->bpp, count);
+        if (count > 4) {
+            av_log(s->avctx, AV_LOG_ERROR,
+                   "This format is not supported (bpp=%d, %d components)\n",
+                   s->bpp, count);
             return -1;
         }
-        if(count == 1) s->bpp = value;
-        else{
-            switch(type){
+        if (count == 1)
+            s->bpp = value;
+        else {
+            switch (type) {
             case TIFF_BYTE:
-                s->bpp = (off & 0xFF) + ((off >> 8) & 0xFF) + ((off >> 16) & 0xFF) + ((off >> 24) & 0xFF);
+                s->bpp = (off & 0xFF) + ((off >> 8) & 0xFF) +
+                         ((off >> 16) & 0xFF) + ((off >> 24) & 0xFF);
                 break;
             case TIFF_SHORT:
             case TIFF_LONG:
                 s->bpp = 0;
-                for(i = 0; i < count && buf < end_buf; i++) s->bpp += tget(&buf, type, s->le);
+                for (i = 0; i < count; i++)
+                    s->bpp += tget(&s->gb, type, s->le);
                 break;
             default:
                 s->bpp = -1;
@@ -373,7 +378,7 @@ static int tiff_decode_tag(TiffContext *s, const uint8_t *start, const uint8_t *
     case TIFF_COMPR:
         s->compr = value;
         s->predictor = 0;
-        switch(s->compr){
+        switch (s->compr) {
         case TIFF_RAW:
         case TIFF_PACKBITS:
         case TIFF_LZW:
@@ -393,56 +398,52 @@ static int tiff_decode_tag(TiffContext *s, const uint8_t *start, const uint8_t *
 #endif
         case TIFF_JPEG:
         case TIFF_NEWJPEG:
-            av_log(s->avctx, AV_LOG_ERROR, "JPEG compression is not supported\n");
+            av_log(s->avctx, AV_LOG_ERROR,
+                   "JPEG compression is not supported\n");
             return -1;
         default:
-            av_log(s->avctx, AV_LOG_ERROR, "Unknown compression method %i\n", s->compr);
+            av_log(s->avctx, AV_LOG_ERROR, "Unknown compression method %i\n",
+                   s->compr);
             return -1;
         }
         break;
     case TIFF_ROWSPERSTRIP:
         if (type == TIFF_LONG && value == UINT_MAX)
             value = s->avctx->height;
-        if(value < 1){
-            av_log(s->avctx, AV_LOG_ERROR, "Incorrect value of rows per strip\n");
+        if (value < 1) {
+            av_log(s->avctx, AV_LOG_ERROR,
+                   "Incorrect value of rows per strip\n");
             return -1;
         }
         s->rps = value;
         break;
     case TIFF_STRIP_OFFS:
-        if(count == 1){
-            s->stripdata = NULL;
+        if (count == 1) {
+            s->strippos = 0;
             s->stripoff = value;
-        }else
-            s->stripdata = start + off;
+        } else
+            s->strippos = off;
         s->strips = count;
-        if(s->strips == 1) s->rps = s->height;
+        if (s->strips == 1)
+            s->rps = s->height;
         s->sot = type;
-        if(s->stripdata > end_buf){
-            av_log(s->avctx, AV_LOG_ERROR, "Tag referencing position outside the image\n");
-            return -1;
-        }
         break;
     case TIFF_STRIP_SIZE:
-        if(count == 1){
-            s->stripsizes = NULL;
-            s->stripsize = value;
-            s->strips = 1;
-        }else{
-            s->stripsizes = start + off;
+        if (count == 1) {
+            s->stripsizesoff = 0;
+            s->stripsize     = value;
+            s->strips        = 1;
+        } else {
+            s->stripsizesoff = off;
         }
         s->strips = count;
         s->sstype = type;
-        if(s->stripsizes > end_buf){
-            av_log(s->avctx, AV_LOG_ERROR, "Tag referencing position outside the image\n");
-            return -1;
-        }
         break;
     case TIFF_PREDICTOR:
         s->predictor = value;
         break;
     case TIFF_INVERT:
-        switch(value){
+        switch (value) {
         case 0:
             s->invert = 1;
             break;
@@ -453,78 +454,88 @@ static int tiff_decode_tag(TiffContext *s, const uint8_t *start, const uint8_t *
         case 3:
             break;
         default:
-            av_log(s->avctx, AV_LOG_ERROR, "Color mode %d is not supported\n", value);
+            av_log(s->avctx, AV_LOG_ERROR, "Color mode %d is not supported\n",
+                   value);
             return -1;
         }
         break;
     case TIFF_FILL_ORDER:
-        if(value < 1 || value > 2){
-            av_log(s->avctx, AV_LOG_ERROR, "Unknown FillOrder value %d, trying default one\n", value);
+        if (value < 1 || value > 2) {
+            av_log(s->avctx, AV_LOG_ERROR,
+                   "Unknown FillOrder value %d, trying default one\n", value);
             value = 1;
         }
         s->fill_order = value - 1;
         break;
-    case TIFF_PAL:
+    case TIFF_PAL: {
+        GetByteContext pal_gb[3];
         pal = (uint32_t *) s->palette;
         off = type_sizes[type];
-        if (count / 3 > 256 || end_buf - buf < count / 3 * off * 3)
+        if (count / 3 > 256 ||
+            bytestream2_get_bytes_left(&s->gb) < count / 3 * off * 3)
             return -1;
-        rp = buf;
-        gp = buf + count / 3 * off;
-        bp = buf + count / 3 * off * 2;
+        pal_gb[0] = pal_gb[1] = pal_gb[2] = s->gb;
+        bytestream2_skip(&pal_gb[1], count / 3 * off);
+        bytestream2_skip(&pal_gb[2], count / 3 * off * 2);
         off = (type_sizes[type] - 1) << 3;
-        for(i = 0; i < count / 3; i++){
-            j = (tget(&rp, type, s->le) >> off) << 16;
-            j |= (tget(&gp, type, s->le) >> off) << 8;
-            j |= tget(&bp, type, s->le) >> off;
-            pal[i] = j;
+        for (i = 0; i < count / 3; i++) {
+            uint32_t p = 0xFF000000;
+            p |= (tget(&pal_gb[0], type, s->le) >> off) << 16;
+            p |= (tget(&pal_gb[1], type, s->le) >> off) << 8;
+            p |=  tget(&pal_gb[2], type, s->le) >> off;
+            pal[i] = p;
         }
         s->palette_is_set = 1;
         break;
+    }
     case TIFF_PLANAR:
-        if(value == 2){
+        if (value == 2) {
             av_log(s->avctx, AV_LOG_ERROR, "Planar format is not supported\n");
             return -1;
         }
         break;
     case TIFF_T4OPTIONS:
-        if(s->compr == TIFF_G3)
+        if (s->compr == TIFF_G3)
             s->fax_opts = value;
         break;
     case TIFF_T6OPTIONS:
-        if(s->compr == TIFF_G4)
+        if (s->compr == TIFF_G4)
             s->fax_opts = value;
         break;
     default:
-        av_log(s->avctx, AV_LOG_DEBUG, "Unknown or unsupported tag %d/0X%0X\n", tag, tag);
+        av_log(s->avctx, AV_LOG_DEBUG, "Unknown or unsupported tag %d/0X%0X\n",
+               tag, tag);
     }
+    bytestream2_seek(&s->gb, start, SEEK_SET);
     return 0;
 }
 
 static int decode_frame(AVCodecContext *avctx,
-                        void *data, int *data_size,
-                        AVPacket *avpkt)
+                        void *data, int *got_frame, AVPacket *avpkt)
 {
-    const uint8_t *buf = avpkt->data;
-    int buf_size = avpkt->size;
-    TiffContext * const s = avctx->priv_data;
+    TiffContext *const s = avctx->priv_data;
     AVFrame *picture = data;
-    AVFrame * const p= (AVFrame*)&s->picture;
-    const uint8_t *orig_buf = buf, *end_buf = buf + buf_size;
+    AVFrame *const p = &s->picture;
     unsigned off;
     int id, le, ret;
     int i, j, entries;
     int stride;
     unsigned soff, ssize;
     uint8_t *dst;
+    GetByteContext stripsizes;
+    GetByteContext stripdata;
+
+    bytestream2_init(&s->gb, avpkt->data, avpkt->size);
 
     //parse image header
-    if (end_buf - buf < 8)
+    if (avpkt->size < 8)
         return AVERROR_INVALIDDATA;
-    id = AV_RL16(buf); buf += 2;
-    if(id == 0x4949) le = 1;
-    else if(id == 0x4D4D) le = 0;
-    else{
+    id = bytestream2_get_le16(&s->gb);
+    if (id == 0x4949)
+        le = 1;
+    else if (id == 0x4D4D)
+        le = 0;
+    else {
         av_log(avctx, AV_LOG_ERROR, "TIFF header not found\n");
         return -1;
     }
@@ -534,26 +545,26 @@ static int decode_frame(AVCodecContext *avctx,
     s->fill_order = 0;
     // As TIFF 6.0 specification puts it "An arbitrary but carefully chosen number
     // that further identifies the file as a TIFF file"
-    if(tget_short(&buf, le) != 42){
-        av_log(avctx, AV_LOG_ERROR, "The answer to life, universe and everything is not correct!\n");
+    if (tget_short(&s->gb, le) != 42) {
+        av_log(avctx, AV_LOG_ERROR,
+               "The answer to life, universe and everything is not correct!\n");
         return -1;
     }
-    // Reset these pointers so we can tell if they were set this frame
-    s->stripsizes = s->stripdata = NULL;
+    // Reset these offsets so we can tell if they were set this frame
+    s->stripsizesoff = s->strippos = 0;
     /* parse image file directory */
-    off = tget_long(&buf, le);
-    if (off >= UINT_MAX - 14 || end_buf - orig_buf < off + 14) {
+    off = tget_long(&s->gb, le);
+    if (off >= UINT_MAX - 14 || avpkt->size < off + 14) {
         av_log(avctx, AV_LOG_ERROR, "IFD offset is greater than image size\n");
         return AVERROR_INVALIDDATA;
     }
-    buf = orig_buf + off;
-    entries = tget_short(&buf, le);
-    for(i = 0; i < entries; i++){
-        if(tiff_decode_tag(s, orig_buf, buf, end_buf) < 0)
+    bytestream2_seek(&s->gb, off, SEEK_SET);
+    entries = tget_short(&s->gb, le);
+    for (i = 0; i < entries; i++) {
+        if (tiff_decode_tag(s) < 0)
             return -1;
-        buf += 12;
     }
-    if(!s->stripdata && !s->stripoff){
+    if (!s->strippos && !s->stripoff) {
         av_log(avctx, AV_LOG_ERROR, "Image data is missing\n");
         return -1;
     }
@@ -561,71 +572,83 @@ static int decode_frame(AVCodecContext *avctx,
     if ((ret = init_image(s)) < 0)
         return ret;
 
-    if(s->strips == 1 && !s->stripsize){
+    if (s->strips == 1 && !s->stripsize) {
         av_log(avctx, AV_LOG_WARNING, "Image data size missing\n");
-        s->stripsize = buf_size - s->stripoff;
+        s->stripsize = avpkt->size - s->stripoff;
     }
     stride = p->linesize[0];
     dst = p->data[0];
-    for(i = 0; i < s->height; i += s->rps){
-        if(s->stripsizes) {
-            if (s->stripsizes >= end_buf)
-                return AVERROR_INVALIDDATA;
-            ssize = tget(&s->stripsizes, s->sstype, s->le);
-        } else
+
+    if (s->stripsizesoff) {
+        if (s->stripsizesoff >= avpkt->size)
+            return AVERROR_INVALIDDATA;
+        bytestream2_init(&stripsizes, avpkt->data + s->stripsizesoff,
+                         avpkt->size - s->stripsizesoff);
+    }
+    if (s->strippos) {
+        if (s->strippos >= avpkt->size)
+            return AVERROR_INVALIDDATA;
+        bytestream2_init(&stripdata, avpkt->data + s->strippos,
+                         avpkt->size - s->strippos);
+    }
+
+    for (i = 0; i < s->height; i += s->rps) {
+        if (s->stripsizesoff)
+            ssize = tget(&stripsizes, s->sstype, le);
+        else
             ssize = s->stripsize;
 
-        if(s->stripdata){
-            if (s->stripdata >= end_buf)
-                return AVERROR_INVALIDDATA;
-            soff = tget(&s->stripdata, s->sot, s->le);
-        }else
+        if (s->strippos)
+            soff = tget(&stripdata, s->sot, le);
+        else
             soff = s->stripoff;
 
-        if (soff > buf_size || ssize > buf_size - soff) {
+        if (soff > avpkt->size || ssize > avpkt->size - soff) {
             av_log(avctx, AV_LOG_ERROR, "Invalid strip size/offset\n");
             return -1;
         }
-        if(tiff_unpack_strip(s, dst, stride, orig_buf + soff, ssize, FFMIN(s->rps, s->height - i)) < 0)
+        if (tiff_unpack_strip(s, dst, stride, avpkt->data + soff, ssize,
+                              FFMIN(s->rps, s->height - i)) < 0)
             break;
         dst += s->rps * stride;
     }
-    if(s->predictor == 2){
+    if (s->predictor == 2) {
         dst = p->data[0];
         soff = s->bpp >> 3;
         ssize = s->width * soff;
-        for(i = 0; i < s->height; i++) {
-            for(j = soff; j < ssize; j++)
+        for (i = 0; i < s->height; i++) {
+            for (j = soff; j < ssize; j++)
                 dst[j] += dst[j - soff];
             dst += stride;
         }
     }
 
-    if(s->invert){
+    if (s->invert) {
         uint8_t *src;
         int j;
 
         src = s->picture.data[0];
-        for(j = 0; j < s->height; j++){
-            for(i = 0; i < s->picture.linesize[0]; i++)
+        for (j = 0; j < s->height; j++) {
+            for (i = 0; i < s->picture.linesize[0]; i++)
                 src[i] = 255 - src[i];
             src += s->picture.linesize[0];
         }
     }
-    *picture= *(AVFrame*)&s->picture;
-    *data_size = sizeof(AVPicture);
+    *picture   = s->picture;
+    *got_frame = 1;
 
-    return buf_size;
+    return avpkt->size;
 }
 
-static av_cold int tiff_init(AVCodecContext *avctx){
+static av_cold int tiff_init(AVCodecContext *avctx)
+{
     TiffContext *s = avctx->priv_data;
 
     s->width = 0;
     s->height = 0;
     s->avctx = avctx;
-    avcodec_get_frame_defaults((AVFrame*)&s->picture);
-    avctx->coded_frame= (AVFrame*)&s->picture;
+    avcodec_get_frame_defaults(&s->picture);
+    avctx->coded_frame = &s->picture;
     ff_lzw_decode_open(&s->lzw);
     ff_ccitt_unpack_init();
 
@@ -634,10 +657,10 @@ static av_cold int tiff_init(AVCodecContext *avctx){
 
 static av_cold int tiff_end(AVCodecContext *avctx)
 {
-    TiffContext * const s = avctx->priv_data;
+    TiffContext *const s = avctx->priv_data;
 
     ff_lzw_decode_close(&s->lzw);
-    if(s->picture.data[0])
+    if (s->picture.data[0])
         avctx->release_buffer(avctx, &s->picture);
     return 0;
 }
@@ -645,11 +668,11 @@ static av_cold int tiff_end(AVCodecContext *avctx)
 AVCodec ff_tiff_decoder = {
     .name           = "tiff",
     .type           = AVMEDIA_TYPE_VIDEO,
-    .id             = CODEC_ID_TIFF,
+    .id             = AV_CODEC_ID_TIFF,
     .priv_data_size = sizeof(TiffContext),
     .init           = tiff_init,
     .close          = tiff_end,
     .decode         = decode_frame,
     .capabilities   = CODEC_CAP_DR1,
-    .long_name = NULL_IF_CONFIG_SMALL("TIFF image"),
+    .long_name      = NULL_IF_CONFIG_SMALL("TIFF image"),
 };
